/**
 * DevExtreme (cjs/__internal/scheduler/workspaces/view_model/m_grouped_data_map_provider.js)
 * Version: 23.2.3 (build 24011-0114)
 * Build date: Thu Jan 11 2024
 *
 * Copyright (c) 2012 - 2024 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.GroupedDataMapProvider = void 0;
var _date = _interopRequireDefault(require("../../../../core/utils/date"));
var _base = require("../../../../renovation/ui/scheduler/view_model/to_test/views/utils/base");
var _date2 = require("../../../core/utils/date");

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    }
}

function _extends() {
    _extends = Object.assign ? Object.assign.bind() : function(target) {
        for (var i = 1; i < arguments.length; i++) {
            var source = arguments[i];
            for (var key in source) {
                if (Object.prototype.hasOwnProperty.call(source, key)) {
                    target[key] = source[key]
                }
            }
        }
        return target
    };
    return _extends.apply(this, arguments)
}
let GroupedDataMapProvider = function() {
    function GroupedDataMapProvider(viewDataGenerator, viewDataMap, completeViewDataMap, viewOptions) {
        this.groupedDataMap = viewDataGenerator.generateGroupedDataMap(viewDataMap);
        this.completeViewDataMap = completeViewDataMap;
        this._viewOptions = viewOptions
    }
    var _proto = GroupedDataMapProvider.prototype;
    _proto.getGroupStartDate = function(groupIndex) {
        var _a, _b, _c;
        const firstRow = this.getFirstGroupRow(groupIndex);
        return null !== (_c = null === (_b = null === (_a = null === firstRow || void 0 === firstRow ? void 0 : firstRow[0]) || void 0 === _a ? void 0 : _a.cellData) || void 0 === _b ? void 0 : _b.startDate) && void 0 !== _c ? _c : null
    };
    _proto.getGroupEndDate = function(groupIndex) {
        const lastRow = this.getLastGroupRow(groupIndex);
        if (lastRow) {
            const lastColumnIndex = lastRow.length - 1;
            const {
                cellData: cellData
            } = lastRow[lastColumnIndex];
            const {
                endDate: endDate
            } = cellData;
            return endDate
        }
    };
    _proto.findGroupCellStartDate = function(groupIndex, startDate, endDate, isFindByDate) {
        const groupData = this.getGroupFromDateTableGroupMap(groupIndex);
        const checkCellStartDate = (rowIndex, columnIndex) => {
            const {
                cellData: cellData
            } = groupData[rowIndex][columnIndex];
            let {
                startDate: secondMin,
                endDate: secondMax
            } = cellData;
            if (isFindByDate) {
                secondMin = _date.default.trimTime(secondMin);
                secondMax = _date.default.setToDayEnd(secondMin)
            }
            if (_date.default.intervalsOverlap({
                    firstMin: startDate,
                    firstMax: endDate,
                    secondMin: secondMin,
                    secondMax: secondMax
                })) {
                return secondMin
            }
        };
        const startDateVerticalSearch = (() => {
            const cellCount = groupData[0].length;
            for (let columnIndex = 0; columnIndex < cellCount; ++columnIndex) {
                for (let rowIndex = 0; rowIndex < groupData.length; ++rowIndex) {
                    const result = checkCellStartDate(rowIndex, columnIndex);
                    if (result) {
                        return result
                    }
                }
            }
        })();
        const startDateHorizontalSearch = (() => {
            for (let rowIndex = 0; rowIndex < groupData.length; ++rowIndex) {
                const row = groupData[rowIndex];
                for (let columnIndex = 0; columnIndex < row.length; ++columnIndex) {
                    const result = checkCellStartDate(rowIndex, columnIndex);
                    if (result) {
                        return result
                    }
                }
            }
        })();
        return startDateVerticalSearch > startDateHorizontalSearch ? startDateHorizontalSearch : startDateVerticalSearch
    };
    _proto.findAllDayGroupCellStartDate = function(groupIndex) {
        var _a, _b, _c;
        const groupedData = this.getGroupFromDateTableGroupMap(groupIndex);
        const cellData = null === (_b = null === (_a = null === groupedData || void 0 === groupedData ? void 0 : groupedData[0]) || void 0 === _a ? void 0 : _a[0]) || void 0 === _b ? void 0 : _b.cellData;
        return null !== (_c = null === cellData || void 0 === cellData ? void 0 : cellData.startDate) && void 0 !== _c ? _c : null
    };
    _proto.findCellPositionInMap = function(cellInfo, isAppointmentRender) {
        const {
            groupIndex: groupIndex,
            startDate: startDate,
            isAllDay: isAllDay,
            index: index
        } = cellInfo;
        const {
            allDayPanelGroupedMap: allDayPanelGroupedMap,
            dateTableGroupedMap: dateTableGroupedMap
        } = this.groupedDataMap;
        const {
            viewOffset: viewOffset
        } = this._viewOptions;
        const rows = isAllDay && !this._viewOptions.isVerticalGrouping ? allDayPanelGroupedMap[groupIndex] ? [allDayPanelGroupedMap[groupIndex]] : [] : dateTableGroupedMap[groupIndex] || [];
        for (let rowIndex = 0; rowIndex < rows.length; rowIndex += 1) {
            const row = rows[rowIndex];
            for (let columnIndex = 0; columnIndex < row.length; columnIndex += 1) {
                const cell = row[columnIndex];
                const cellData = isAppointmentRender ? _extends(_extends({}, cell.cellData), {
                    startDate: _date2.dateUtilsTs.addOffsets(cell.cellData.startDate, [-viewOffset]),
                    endDate: _date2.dateUtilsTs.addOffsets(cell.cellData.endDate, [-viewOffset])
                }) : cell.cellData;
                if (this._isSameGroupIndexAndIndex(cellData, groupIndex, index)) {
                    if (this.isStartDateInCell(startDate, isAllDay, cellData)) {
                        return cell.position
                    }
                }
            }
        }
        return
    };
    _proto.isStartDateInCell = function(startDate, inAllDayRow, _ref) {
        let {
            startDate: cellStartDate,
            endDate: cellEndDate,
            allDay: cellAllDay
        } = _ref;
        const {
            viewType: viewType
        } = this._viewOptions;
        switch (true) {
            case !(0, _base.isDateAndTimeView)(viewType):
            case inAllDayRow && cellAllDay:
                return _date.default.sameDate(startDate, cellStartDate);
            case !inAllDayRow:
                return startDate >= cellStartDate && startDate < cellEndDate;
            default:
                return false
        }
    };
    _proto._isSameGroupIndexAndIndex = function(cellData, groupIndex, index) {
        return cellData.groupIndex === groupIndex && (void 0 === index || cellData.index === index)
    };
    _proto.getCellsGroup = function(groupIndex) {
        const {
            dateTableGroupedMap: dateTableGroupedMap
        } = this.groupedDataMap;
        const groupData = dateTableGroupedMap[groupIndex];
        if (groupData) {
            const {
                cellData: cellData
            } = groupData[0][0];
            return cellData.groups
        }
    };
    _proto.getCompletedGroupsInfo = function() {
        const {
            dateTableGroupedMap: dateTableGroupedMap
        } = this.groupedDataMap;
        return dateTableGroupedMap.map(groupData => {
            const firstCell = groupData[0][0];
            const {
                allDay: allDay,
                groupIndex: groupIndex
            } = firstCell.cellData;
            return {
                allDay: allDay,
                groupIndex: groupIndex,
                startDate: this.getGroupStartDate(groupIndex),
                endDate: this.getGroupEndDate(groupIndex)
            }
        }).filter(_ref2 => {
            let {
                startDate: startDate
            } = _ref2;
            return !!startDate
        })
    };
    _proto.getGroupIndices = function() {
        return this.getCompletedGroupsInfo().map(_ref3 => {
            let {
                groupIndex: groupIndex
            } = _ref3;
            return groupIndex
        })
    };
    _proto.getGroupFromDateTableGroupMap = function(groupIndex) {
        const {
            dateTableGroupedMap: dateTableGroupedMap
        } = this.groupedDataMap;
        return dateTableGroupedMap[groupIndex]
    };
    _proto.getFirstGroupRow = function(groupIndex) {
        const groupedData = this.getGroupFromDateTableGroupMap(groupIndex);
        if (groupedData) {
            const {
                cellData: cellData
            } = groupedData[0][0];
            return !cellData.allDay ? groupedData[0] : groupedData[1]
        }
    };
    _proto.getLastGroupRow = function(groupIndex) {
        const {
            dateTableGroupedMap: dateTableGroupedMap
        } = this.groupedDataMap;
        const groupedData = dateTableGroupedMap[groupIndex];
        if (groupedData) {
            const lastRowIndex = groupedData.length - 1;
            return groupedData[lastRowIndex]
        }
    };
    _proto.getLastGroupCellPosition = function(groupIndex) {
        const groupRow = this.getLastGroupRow(groupIndex);
        return null === groupRow || void 0 === groupRow ? void 0 : groupRow[(null === groupRow || void 0 === groupRow ? void 0 : groupRow.length) - 1].position
    };
    _proto.getRowCountInGroup = function(groupIndex) {
        const groupRow = this.getLastGroupRow(groupIndex);
        const cellAmount = groupRow.length;
        const lastCellData = groupRow[cellAmount - 1].cellData;
        const lastCellIndex = lastCellData.index;
        return (lastCellIndex + 1) / groupRow.length
    };
    return GroupedDataMapProvider
}();
exports.GroupedDataMapProvider = GroupedDataMapProvider;
